import urwid
from babel import numbers


class Value(urwid.Text):

    def __init__(self, value, currency):
        rendered_text = self.render_value(value, currency)
        super().__init__(rendered_text, 'right')

    def render_value(self, value, currency):
        return numbers.format_currency(value, currency)



import urwid


class Popup(urwid.Overlay):

    def __init__(self, loop, *args, **kwargs):
        self.old_widget = None
        self.loop = loop

        widget = self.get_ui(args, kwargs)

        super().__init__(widget,
                         loop.widget,
                         align=("relative", 50),
                         valign=("relative", 25),
                         width=("relative", 20),
                         height=6)

    def get_ui(self, *args, **kwargs):
        raise NotImplemented

    def start(self):
        self.old_widget = self.loop.widget
        self.loop.widget = self

    def exit(self, *args, **kwargs):
        self.loop.widget = self.old_widget



import pytest

from cryptopcash.market.CryptoCompare import CryptoCompare
from cryptopcash.model.Market import CryptoCurrency


def test_get_coins_list():
    crypto_compare = CryptoCompare()
    coins_list = list(crypto_compare.get_coins_list())

    assert coins_list
    assert len(coins_list) == 3

    assert coins_list[0].symbol == 'BTC'
    assert coins_list[1].symbol == 'LTC'
    assert coins_list[2].symbol == 'IOT'


def test_get_price():
    crypto_compare = CryptoCompare()

    btc = CryptoCurrency('Bitcoin', 'BTC')
    price = crypto_compare.get_coin_price(btc, 'EUR')
    assert price
    assert price.coin.symbol == 'BTC'
    assert price.price == 7000.00
    assert price.low == 6800.00
    assert price.high == 7200.00


@pytest.mark.parametrize("symbols", [
    ['BTC'],
    ['BTC', 'LTC'],
    ['IOT', 'ETH'],
])
def test_get_prices(symbols):
    coins = [CryptoCurrency('Bitcoin', symbol) for symbol in symbols]

    crypto_compare = CryptoCompare()
    prices = list(crypto_compare.get_coins_prices(coins, 'EUR'))

    assert prices
    assert len(prices) == len(coins)
    for price in prices:
        assert price.coin.symbol in symbols
        assert price.price == 7000.00
        assert price.low == 6800.00
        assert price.high == 7200.00



from cryptopcash.market.CryptoCompare import CryptoCompare

from cryptopcash.model.Wallet import Wallet, Holding
from cryptopcash.model.Market import Market
from cryptopcash.model.Config import Config

from cryptopcash.ui.Main import Main

from cryptopcash.storage.LocalStorage import LocalStorage


class CryptopCash(object):

    instance = None

    def __init__(self):
        self.wallet = Wallet()
        self.market = Market()
        self.config = Config()

        # load config file
        self.config.load()

        # get Holdings from data file
        local_storage = LocalStorage()
        holdings = local_storage.load_holdings()
        for holding in holdings:
            self.wallet.add_holding(holding)

        c = CryptoCompare()
        # Update coin information
        for holding in self.wallet.holdings:
            symbol = holding.coin.symbol
            holding.coin = c.get_coin_info(symbol)

        # Get price
        self.update_price()

    def start_cryptopcash(self):
        self.main_ui = Main(self.wallet, self.market, self.config)
        self.main_ui.run()

    @staticmethod
    def get_instance():
        if not CryptopCash.instance:
            CryptopCash.instance = CryptopCash()
        return CryptopCash.instance

    def add_holding(self, coin, quantity):
        holding = Holding(coin, quantity)
        self.wallet.add_holding(holding)

        local_storage = LocalStorage()
        local_storage.save_holdings(self.wallet.holdings)
        self.refresh()

    def get_coin_info(self, symbol):
        c = CryptoCompare()
        return c.get_coin_info(symbol)

    def refresh(self):
        self.update_price()
        self.main_ui.refresh()

    def update_price(self):
        c = CryptoCompare()
        holdings = self.wallet.holdings
        coins = [holding.coin for holding in holdings]
        if coins:
            for price in c.get_coins_prices(coins, self.config.currency):
                self.market.add_currency(price)


def main():
    cryptopcash = CryptopCash.get_instance()
    cryptopcash.start_cryptopcash()


if __name__ == '__main__':
    main()



import pytest
from cryptocompy import coin as cryptocompy_coin
from cryptocompy import price as cryptocompy_price


PRICES = {'BTC': (8000, 9000, 7000),
          'LTC': (500, 600, 400),
          'IOT': (5, 6, 3)}


@pytest.fixture(autouse=True)
def cryptocompare_mock(monkeypatch):

    monkeypatch.setattr(cryptocompy_coin, "get_coin_list", get_coin_list)
    monkeypatch.setattr(cryptocompy_price, "get_current_price", get_price)


def get_coin_list(coins=None):
    res = {'BTC':
           {'Algorithm': 'SHA256',
            'CoinName': 'Bitcoin',
            'FullName': 'Bitcoin (BTC)',
            'FullyPremined': '0',
            'Id': '1182',
            'ImageUrl': '/media/19633/btc.png',
            'Name': 'BTC',
            'PreMinedValue': 'N/A',
            'ProofType': 'PoW',
            'SortOrder': '1',
            'Sponsored': False,
            'Symbol': 'BTC',
            'TotalCoinSupply': '21000000',
            'TotalCoinsFreeFloat': 'N/A',
            'Url': '/coins/btc/overview'},
           'LTC':
           {'Algorithm': 'Scrypt',
            'CoinName': 'Litecoin',
            'FullName': 'Litecoin (LTC)',
            'FullyPremined': '0',
            'Id': '3808',
            'ImageUrl': '/media/19782/litecoin-logo.png',
            'Name': 'LTC',
            'PreMinedValue': 'N/A',
            'ProofType': 'PoW',
            'SortOrder': '3',
            'Sponsored': False,
            'Symbol': 'LTC',
            'TotalCoinSupply': '84000000',
            'TotalCoinsFreeFloat': 'N/A',
            'Url': '/coins/ltc/overview'},
           'IOT':
           {'Algorithm': 'N/A',
            'CoinName': 'IOTA',
            'FullName': 'IOTA (IOT)',
            'FullyPremined': '1',
            'Id': '127356',
            'ImageUrl': '/media/1383540/iota_logo.png',
            'Name': 'IOT',
            'PreMinedValue': 'N/A',
            'ProofType': 'Tangle',
            'SortOrder': '1247',
            'Sponsored': False,
            'Symbol': 'IOT',
            'TotalCoinSupply': '2779530283',
            'TotalCoinsFreeFloat': 'N/A',
            'Url': '/coins/iot/overview'}}
    if not coins:
        return res

    if not isinstance(coins, (list, tuple)):
        coins = [coins]
    return {coin: res[coin] for coin in coins if coin in res.keys()}


def raw_price_template(price=7000.00, low=6800.00, high=7200.00):
    return {'CHANGE24HOUR': -297.72999999999956,
            'CHANGEDAY': 10.510000000000218,
            'CHANGEPCT24HOUR': -4.11660843342675,
            'CHANGEPCTDAY': 0.1517871456073467,
            'FLAGS': '1',
            'FROMSYMBOL': 'BTC',
            'HIGH24HOUR': high,
            'HIGHDAY': 7017.99,
            'LASTMARKET': 'Kraken',
            'LASTTRADEID': '1521994152.0339',
            'LASTUPDATE': 1521994152,
            'LASTVOLUME': 0.0131976,
            'LASTVOLUMETO': 91.52667576,
            'LOW24HOUR': low,
            'LOWDAY': 6811.89,
            'MARKET': 'CCCAGG',
            'MKTCAP': 117465330951,
            'OPEN24HOUR': 7232.41,
            'OPENDAY': 6924.17,
            'PRICE': price,
            'SUPPLY': 16938825,
            'TOSYMBOL': 'EUR',
            'TOTALVOLUME24H': 420193.2637329436,
            'TOTALVOLUME24HTO': 2914491767.1551404,
            'TYPE': '5',
            'VOLUME24HOUR': 9936.121144569997,
            'VOLUME24HOURTO': 69489765.59039718,
            'VOLUMEDAY': 5612.430257430866,
            'VOLUMEDAYTO': 38821131.9139348}


def display_price_template(price='€ 7,000.00', low='€ 6,800.00',
                           high='€ 7,200.00'):
    return {'CHANGE24HOUR': '€ -297.73',
            'CHANGEDAY': '€ 10.51',
            'CHANGEPCT24HOUR': '-4.12',
            'CHANGEPCTDAY': '0.15',
            'FROMSYMBOL': 'Ƀ',
            'HIGH24HOUR': high,
            'HIGHDAY': '€ 7,017.99',
            'LASTMARKET': 'Kraken',
            'LASTTRADEID': '1521994152.0339',
            'LASTUPDATE': 'Just now',
            'LASTVOLUME': 'Ƀ 0.01320',
            'LASTVOLUMETO': '€ 91.53',
            'LOW24HOUR': low,
            'LOWDAY': '€ 6,811.89',
            'MARKET': 'CryptoCompare Index',
            'MKTCAP': '€ 117.47 B',
            'OPEN24HOUR': '€ 7,232.41',
            'OPENDAY': '€ 6,924.17',
            'PRICE': price,
            'SUPPLY': 'Ƀ 16,938,825.0',
            'TOSYMBOL': '€',
            'TOTALVOLUME24H': 'Ƀ 420.19 K',
            'TOTALVOLUME24HTO': '€ 2,914.49 M',
            'VOLUME24HOUR': 'Ƀ 9,936.12',
            'VOLUME24HOURTO': '€ 69,489,765.6',
            'VOLUMEDAY': 'Ƀ 5,612.43',
            'VOLUMEDAYTO': '€ 38,821,131.9'}


def get_price(symbols, unit, full=False):
    if not isinstance(symbols, (list, tuple)):
        symbols = [symbols]

    raw_prices = {}
    for symbol in symbols:
        raw_prices[symbol] = {unit: raw_price_template()}

    return raw_prices





class Wallet(object):

    def __init__(self):
        self.holdings = []

    def add_holding(self, holding):
        coin = holding.coin
        new_quantity = holding.holding
        existing_holding = [holding for holding in self.holdings
                            if holding.coin == coin]
        if existing_holding:
            existing_holding[0].holding += new_quantity
        else:
            holding = Holding(coin, new_quantity)
            self.holdings.append(holding)

    def total_holding_value(self, market):
        total = 0.0
        for holding in self.holdings:
            holding_value = holding.get_holding_value(market)
            total += holding_value
        return total


class Holding(object):

    def __init__(self, coin, holding):
        self.coin = coin
        self.holding = holding

    def get_holding_value(self, market):
        currency_price = market.get_currency_price(self.coin.symbol)
        return float(self.holding) * float(currency_price.price)



import urwid

from cryptopcash.ui.Asset import Asset
from cryptopcash.ui.Title import Title
from cryptopcash.ui.Header import Header
from cryptopcash.ui.TotalHolding import TotalHolding
from cryptopcash.ui.AddHoldingDialog import AddHoldingDialog
from cryptopcash.ui.ErrorDialog import ErrorDialog, DialogException
from cryptopcash.ui.AssetsView import AssetsView


class Main(object):

    def __init__(self, wallet, market, conf):

        def show_or_exit(key):
            if key in ('q', 'Q'):
                raise urwid.ExitMainLoop()
            if key == 'a':
                dialog = AddHoldingDialog(self.loop)
                dialog.start()

        palette = self.get_palette(conf)
        main_ui = self.create_ui(wallet, market, conf)
        self.loop = urwid.MainLoop(main_ui, palette,
                                   unhandled_input=show_or_exit)

    def create_ui(self, wallet, market, conf):

        title = Title()
        header = Header()
        assets = [Asset(holding, market, conf) for holding in wallet.holdings]
        self.assets_view = AssetsView(assets)
        total = TotalHolding(wallet, market, conf)

        contents_view = urwid.LineBox(urwid.Frame(self.assets_view, header=header))
        main_view = urwid.Frame(contents_view, header=title, footer=total)

        return urwid.AttrMap(main_view, 'default')

    def get_palette(self, config):
        return [('default', config.text, 'black'),
                ('highlighted', config.banner_text, config.banner)]

    def run(self):

        while True:
            try:
                self.loop.run()
            except DialogException as e:
                # TODO Remove the popup. May be handled better
                self.loop.widget.exit()
                dialog = ErrorDialog(self.loop, e)
                dialog.start()
                continue
            break

    def refresh(self):
        from cryptopcash.cryptopcash import CryptopCash
        context = CryptopCash.get_instance()

        wallet = context.wallet
        market = context.market
        conf = context.config

        new_assets = [Asset(holding, market, conf) for holding in wallet.holdings]
        self.assets_view.refresh_assets(new_assets)



import urwid

import cryptopcash


class Title(urwid.Columns):

    def __init__(self, pallette='default'):
        text_widget = urwid.Text(self.render_value())
        super().__init__([text_widget])

    def render_value(self):
        version = cryptopcash.__version__
        return f"cryptocash v{version}"



import os
import shutil
import xdg
import pytest
import tempfile
import json
from configparser import ConfigParser

from cryptopcash.storage import LocalStorage


@pytest.fixture(autouse=True)
def locale_storage(monkeypatch):
    config_dir = tempfile.mkdtemp()
    cryptopcash_config = os.path.join(config_dir,
                                      LocalStorage.PREFIX_DIRECTORY_NAME)
    os.mkdir(cryptopcash_config)

    data_dir = tempfile.mkdtemp()
    cryptopcash_data = os.path.join(data_dir,
                                    LocalStorage.PREFIX_DIRECTORY_NAME)
    os.mkdir(cryptopcash_data)

    monkeypatch.setattr(xdg, "XDG_CONFIG_HOME", config_dir)
    monkeypatch.setattr(xdg, "XDG_DATA_HOME", data_dir)

    os.mknod(os.path.join(cryptopcash_data, LocalStorage.WALLET_FILENAME))
    os.mknod(os.path.join(cryptopcash_config, LocalStorage.CONFIG_FILENAME))

    yield LocalStorage.LocalStorage()

    shutil.rmtree(config_dir)
    shutil.rmtree(data_dir)


@pytest.fixture()
def data_file(locale_storage, request):
    content = request.param
    data_filename = locale_storage.get_locale_data_filename()
    with open(data_filename, 'w') as fd:
        json.dump(content, fd)


@pytest.fixture()
def config_file_sample(locale_storage):

    config = ConfigParser()

    config['theme'] = {'text': 'black'}

    config['api'] = {'currency': 'YEN'}

    config_filename = locale_storage.get_locale_config_filename()
    with open(config_filename, 'w') as fd:
        config.write(fd)



import os
import pytest

from cryptopcash.storage.LocalStorage import LocalStorage


def test_get_locale_config():
    locale_storage = LocalStorage()
    local_config_filename = locale_storage.get_locale_config_filename()
    assert local_config_filename
    assert os.path.exists(local_config_filename)


def test_get_locale_data():
    locale_storage = LocalStorage()
    local_data_filename = locale_storage.get_locale_data_filename()
    assert local_data_filename
    assert os.path.exists(local_data_filename)


@pytest.mark.parametrize('data_file', [
    {'LTC': 40, 'BTC': 50}],
    indirect=True)
def test_load_holdings(data_file):
    locale_storage = LocalStorage()
    holdings = locale_storage.load_holdings()

    assert holdings
    assert len(holdings) == 2
    assert holdings[0].coin.symbol == 'LTC'
    assert holdings[0].holding == 40.0
    assert holdings[1].coin.symbol == 'BTC'
    assert holdings[1].holding == 50.0


def test_get_holdings_no_wallet():
    locale_storage = LocalStorage()
    assert not locale_storage.load_holdings()


def test_get_default_config_filename():
    locale_storage = LocalStorage()
    default_config = locale_storage.get_default_config_filename()
    assert default_config.exists()



from pathlib import Path


def get_package_path():
    import cryptopcash
    cryptopcash_path = Path(cryptopcash.__file__)
    return cryptopcash_path.parents[1]


def get_resource_path():
    return get_package_path() / 'resources'



from cryptopcash.model.Config import Config


def test_config(config_file_sample):
    config = Config()
    config.load()

    assert config.currency == 'YEN'
    assert config.text == 'black'

    assert config.banner == 'yellow'
    assert config.banner_text == 'black'

    assert not config.monetary



import urwid


HEADERS = [("COIN", 'left'),
           ("PRICE", 'right'),
           ("HELD", 'right'),
           ("VAL", 'right'),
           ("HIGH", 'right'),
           ("LOW", 'right')]


class Header(urwid.AttrMap):

    def __init__(self, palette='highlighted'):
        columns = [urwid.Text(txt, alignment) for txt, alignment in HEADERS]
        row = urwid.Columns(columns)
        super().__init__(row, palette)



from urwid.display_common import _BASIC_COLORS as urwid_colors

from cryptopcash.storage.LocalStorage import LocalStorage


class InvalidColor(Exception):
    pass


class Config(object):

    def __init__(self):
        # Theme
        self.text = 'yellow'
        self.banner = 'yellow'
        self.banner_text = 'black'
        self.background = -1
        self.dec_places = 2
        self.field_length = 12

        # API
        self.currency = 'USD'
        self.cache = 10

        # Locale
        self.monetary = None

    def load(self):
        storage = LocalStorage()
        parser = storage.get_config()

        self.load_section(parser, 'theme', ['text', 'banner', 'banner_text',
                                            'background', 'dec_places',
                                            'field_length'])
        self.load_section(parser, 'api', ['currency', 'cache'])
        self.load_section(parser, 'locale', ['monetary'])

        self.validate()

    def load_section(self, parser, section_name, attributes):
        try:
            api_config = parser[section_name]
        except KeyError:
            return

        for attribute in attributes:
            attribute_value = api_config.get(attribute, None)
            if attribute_value:
                self.__setattr__(attribute, attribute_value)

    def validate(self):
        if self.text not in urwid_colors:
            raise InvalidColor("{} color is not available".format(self.text))
        if self.banner not in urwid_colors:
            raise InvalidColor("{} color is not available".format(self.banner))
        if self.banner_text not in urwid_colors:
            raise InvalidColor("{} color is not available".format(self.banner_text))



import urwid

from cryptopcash.ui.Popup import Popup


class DialogException(Exception):

    def __init__(self, text):
        self.text = text


class ErrorDialog(Popup):

    def __init__(self, loop, exception, *args, **kwargs):
        self.text = exception.text

        Popup.__init__(self, loop, args, kwargs)

    def get_ui(self, *args, **kwargs):

        exit_button = urwid.Button('Done')
        error_text = urwid.Text(self.text)

        text_column = urwid.Columns([error_text])
        button_column = urwid.Columns([exit_button])
        pile = urwid.Pile([text_column, button_column])

        self.widget = urwid.Filler(pile, 'top')

        urwid.connect_signal(exit_button, 'click', self.exit)

        return self.widget



import pytest

from cryptopcash.model.Market import CryptoCurrency


@pytest.mark.parametrize("param_a, param_b, res", [
    ({'name': 'Bitcoin', 'symbol': 'BTC'}, {'name': 'err', 'symbol': 'BTC'}, True),
    ({'name': 'Bitcoin', 'symbol': 'BTC'}, {'name': 'Bitcoin', 'symbol': 'BCH'}, False),
 ])
def test_equal_coin(param_a, param_b, res):
    coin_a = CryptoCurrency(**param_a)
    coin_b = CryptoCurrency(**param_b)

    assert (coin_a == coin_b) == res



from cryptocompy import coin as cryptocompy_coin
from cryptocompy import price as cryptocompy_price

from cryptopcash.market.Market import Market

from cryptopcash.model.Market import CryptoCurrency, CurrencyPrice

from cryptopcash.ui.ErrorDialog import DialogException


class UnknownCoin(DialogException):
    def __init__(self):
        DialogException.__init__(self, "This coin doesn't exist on CryptoCompare.com market")


class CryptoCompare(Market):

    def get_coins_list(self):
        res = cryptocompy_coin.get_coin_list()
        for symbol, coins in res.items():
            name = coins['CoinName']
            yield CryptoCurrency(name, symbol)

    def get_coin_info(self, symbol):
        try:
            infos = cryptocompy_coin.get_coin_list(coins=symbol)
        except KeyError:
            raise UnknownCoin
        name = infos[symbol]['CoinName']
        return CryptoCurrency(name, symbol)

    def get_coin_price(self, coin, unit):
        symbol = coin.symbol

        res = cryptocompy_price.get_current_price(symbol, unit, full=True)
        infos = res[symbol][unit]
        price, high, low = self.prices_from_res_infos(infos)

        return CurrencyPrice(coin, price, low, high)

    def get_coins_prices(self, coins, unit):
        symbols = [coin.symbol for coin in coins]
        res = cryptocompy_price.get_current_price(symbols, unit, full=True)

        for symbol, info in res.items():
            price, high, low = self.prices_from_res_infos(info[unit])
            coin = [c for c in coins if c.symbol == symbol]
            yield CurrencyPrice(coin[0], price, low, high)

    def prices_from_res_infos(self, infos):
        price = infos['PRICE']
        high = infos['HIGH24HOUR']
        low = infos['LOW24HOUR']

        return price, high, low



import urwid

from cryptopcash.ui.Value import Value


class Asset(urwid.Columns):

    def __init__(self, holding, market, conf):

        marked_price = market.get_currency_price(holding.coin.symbol)

        unit = conf.currency
        currency = urwid.Text(marked_price.coin.name, 'left')
        price = Value(marked_price.price, unit)
        held = urwid.Text(str(holding.holding), 'right')
        value = Value(holding.get_holding_value(market), unit)

        high = Value(marked_price.high, unit)
        low = Value(marked_price.low, unit)

        super().__init__([currency,
                          price,
                          held,
                          value,
                          high,
                          low])





class Market(object):

    def __init__(self):
        self.currencies = {}

    def add_currency(self, currency):
        self.currencies[currency.coin.symbol] = currency

    def get_currency_price(self, symbol):
        return self.currencies.get(symbol, None)


class CryptoCurrency(object):

    def __init__(self, name, symbol):
        self.name = name
        self.symbol = symbol

    def __eq__(self, other):
        return self.symbol == other.symbol


class CurrencyPrice(object):

    def __init__(self, coin, price, low, high):
        self.coin = coin
        self.price = price
        self.low = low
        self.high = high



import pytest

from cryptopcash.model.Wallet import Wallet, Holding
from cryptopcash.model.Market import CryptoCurrency


@pytest.mark.parametrize("adds, result", [
    ([],
     []),

    ([('BTC', 30)],
     [('BTC', 30)]),

    ([('BTC', 30), ('BTC', 20)],
     [('BTC', 50)]),

    ([('BTC', 30), ('LTC', 42), ('BTC', 20)],
     [('BTC', 50), ('LTC', 42)]),
])
def test_add_holding(adds, result):
    wallet = Wallet()

    for symbol, quantity in adds:
        coin = CryptoCurrency(symbol, symbol)
        holding = Holding(coin, quantity)
        wallet.add_holding(holding)

    holdings = wallet.holdings
    crypto_held = [symbol for symbol, _ in result]
    result_held = [holding.coin.symbol for holding in holdings]

    assert set(result_held) == set(crypto_held)

    for symbol, quantity in result:
        holding = [holding.holding for holding in holdings if holding.coin.symbol == symbol]
        assert holding[0] == quantity



import urwid

from cryptopcash.ui.Popup import Popup


class AddHoldingDialog(Popup):

    def get_ui(self, *args, **kwargs):
        self.symbol = urwid.Edit("Symbol :")
        self.holding = urwid.Edit("Holding :")
        exit_button = urwid.Button('Exit')
        submit_button = urwid.Button('Add')

        edit_column = urwid.Columns([self.symbol, self.holding])
        button_column = urwid.Columns([submit_button, exit_button])
        pile = urwid.Pile([edit_column, button_column])

        self.widget = urwid.Filler(pile, 'top')

        urwid.connect_signal(exit_button, 'click', self.exit)
        urwid.connect_signal(submit_button, 'click', self.add_holding)

        return self.widget

    def add_holding(self, button):
        from cryptopcash.cryptopcash import CryptopCash
        context = CryptopCash.get_instance()

        symbol = self.symbol.get_edit_text()
        coin = context.get_coin_info(symbol)
        quantity = float(self.holding.get_edit_text())

        context.add_holding(coin, quantity)
        self.exit()



"""
test about CryptopCash class
"""
import pytest
from mock import MagicMock

from cryptopcash.cryptopcash import CryptopCash
from cryptopcash.model.Market import CryptoCurrency


def test_instance():
    assert not CryptopCash.instance
    first_instance = CryptopCash.get_instance()

    next_instance = CryptopCash.get_instance()
    assert next_instance == first_instance


@pytest.mark.parametrize('data_file', [
    {'LTC': 40, 'BTC': 50}],
    indirect=True)
def test_load_holding(data_file):
    cryptopcash = CryptopCash.get_instance()

    holdings = cryptopcash.wallet.holdings

    assert holdings
    assert len(holdings) == 2
    assert holdings[0].coin.symbol == 'LTC'
    assert holdings[0].holding == 40.0
    assert holdings[1].coin.symbol == 'BTC'
    assert holdings[1].holding == 50.0


def test_add_holding():
    cryptopcash = CryptopCash.get_instance()
    cryptopcash.main_ui = MagicMock()

    coin = CryptoCurrency('Bitcoin', 'BTC')
    cryptopcash.add_holding(coin, 50)

    holdings = cryptopcash.wallet.holdings
    assert holdings
    assert len(holdings) == 1



import urwid


class AssetsView(urwid.Pile):

    def __init__(self, assets):
        contents = [('fixed', 1, urwid.Filler(asset)) for asset in assets]
        self.content_view = urwid.Pile(contents)

        super().__init__(contents)

    def refresh_assets(self, new_assets):
        new_contents = [(urwid.Filler(asset), ('given', 1)) for asset in new_assets]
        self.contents = new_contents



import urwid
from babel import numbers


class TotalHolding(urwid.AttrMap):

    def __init__(self, wallet, market, conf, palette='highlighted'):
        holdings_string = self.total_holding_str(wallet, market, conf)
        text_widget = urwid.Text(holdings_string)
        column = urwid.Columns([text_widget])

        super().__init__(column, palette)

    def total_holding_str(self, wallet, market, conf):
        total_value = wallet.total_holding_value(market)
        str_total_value = numbers.format_currency(total_value, conf.currency)
        return "Total Holdings: {}".format(str_total_value)



import os
import xdg
import json
import logging
from configparser import ConfigParser
from pathlib import Path
import shutil

from cryptopcash.model.Wallet import Holding
from cryptopcash.model.Market import CryptoCurrency
from cryptopcash import util


PREFIX_DIRECTORY_NAME = 'cryptopcash'
CONFIG_FILENAME = "config.ini"
WALLET_FILENAME = "wallet.json"


class LocalStorage(object):

    def get_locale_config_filename(self):
        return os.path.join(xdg.XDG_CONFIG_HOME,
                            PREFIX_DIRECTORY_NAME,
                            CONFIG_FILENAME)

    def get_locale_data_filename(self):
        return os.path.join(xdg.XDG_DATA_HOME,
                            PREFIX_DIRECTORY_NAME,
                            WALLET_FILENAME)

    def load_holdings(self):
        wallet_filename = self.get_locale_data_filename()

        if not os.path.exists(wallet_filename):
            return []

        try:
            with open(wallet_filename, 'r') as fd:
                holding_data = json.load(fd)
        except (FileNotFoundError, ValueError):
            logging.error("date file {} not readable".format(wallet_filename))
            return []

        holdings = []
        for currency, holding in holding_data.items():
            coin = CryptoCurrency("", currency)
            holding = Holding(coin, float(holding))
            holdings.append(holding)

        return holdings

    def save_holdings(self, holdings):
        wallet_filename = self.get_locale_data_filename()

        json_holdings = {held.coin.symbol: held.holding for held in holdings}

        try:
            with open(wallet_filename, 'w+') as fd:
                json.dump(json_holdings, fd)
        except (FileNotFoundError, ValueError):
            logging.error("date file {} not readable".format(wallet_filename))
            return []

    def get_config(self):
        config_filename = self.get_locale_config_filename()
        if not Path(config_filename).exists():
            self.copy_default_config(config_filename)

        parser = ConfigParser()
        parser.read(config_filename)
        return parser

    def get_default_config_filename(self):
        return util.get_resource_path() / 'config' / CONFIG_FILENAME

    def copy_default_config(self, config_filename):
        default_config_filename = self.get_default_config_filename()

        config_path = Path(config_filename).parents[0]
        if not config_path.exists():
            config_path.mkdir()

        shutil.copyfile(default_config_filename, config_filename)
